## code to prepare `DATASET` dataset goes here

region_at_time_txtfls = function(filename){
  # function returning the mapping from one region name and code tag to the next according to the SSB region classification
  #Fylker: https://www.ssb.no/en/klass/klassifikasjoner/104/versjon/1158/koder
  #Kommuner: https://www.ssb.no/en/klass/klassifikasjoner/131

  #   filename = files[1]
  #readr::guess_encoding(filename)
  datastring = readLines(filename, n=-1L,
                         encoding =  dplyr::pull(readr::guess_encoding(filename)[1,1]),
                         warn = F)#nchars = 10^6)
  headings = unlist(stringr::str_split(datastring[1], "\t"))
  print(headings)
  datastring = datastring[-1]

  if((length(headings)%% 2) == 0 & length(datastring)>1) {
    # Then string should be arranged to pairs of "froms" and "tos"
    datastring = stringr::str_remove(datastring, pattern = "\t")
    convtable = matrix(data = datastring, ncol = 2, byrow = T)
    colnames(convtable) = headings
    headingsinv = unlist(lapply(X= str_split(headings, " "), FUN = function(X){paste0(X[2]," ",  X[1], " 1")}))

    colnamecandidates = stringr::str_sub(
      stringr::str_replace_all(
        string = lubridate::ymd(headingsinv),
        pattern = "-",
        replacement = ""),
      start=1, end = 6)

    convtable = dplyr::as_tibble(as.data.frame(convtable, stringsAsFactors=F))
    froms =  dplyr::as_tibble(
      str_split(
        string =  dplyr::pull(convtable[,1]), pattern = " - ", n=2, simplify = T))
    colnames(froms) = paste(c("reg_code", "reg_name"), rep(colnamecandidates[1], 2), sep = "_")

    tos = dplyr::as_tibble(
      stringr::str_split(
        string =  dplyr::pull(convtable[,2]), pattern = " - ", n=2, simplify = T))
    colnames(tos) = paste(c("reg_code", "reg_name"), rep(colnamecandidates[2], 2), sep = "_")

    fromstos = dplyr::bind_cols(froms, tos)
  } else if((length(headings)%% 2) == 1 & length(datastring)>1) {
    # Then it is the starting point, i.e, first array of region units

    convtable = matrix(data = datastring, ncol = 1, byrow = T)
    #colnames(convtable) = headings
    headingsinv =
      unlist(lapply(X= stringr::str_split(headings, " "),
                    FUN = function(X){ paste0(X[2]," ",  X[1], " 1")
                    }))
    colnamecandidates =
      stringr::str_sub(
        stringr::str_replace_all(
          string =  lubridate::ymd(headingsinv),
          pattern = "-",
          replacement = ""), start=1, end = 6)

    convtable = as.data.frame(convtable, stringsAsFactors=F )
    froms =  data.frame(
      stringr::str_split(
        string =  convtable[,1],
        pattern = " - ",
        n=2,
        simplify = T), stringsAsFactors = F )

    colnames(froms) = paste(c("reg_code", "reg_name"), rep(colnamecandidates[1], 2), sep = "_")

    fromstos = dplyr::as_tibble(froms)

  } else  if((length(headings)%% 2) == 0 & length(datastring) == 0) { # THen it is an empty update but we still need the "update dates"

    headingsinv = unlist(lapply(X= str_split(headings, " "), FUN = function(X){paste0(X[2]," ",  X[1], " 1")}))
    colnamecandidates = stringr::str_sub(
      stringr::str_replace_all(
        string =  lubridate::ymd(headingsinv),
        pattern = "-",
        replacement = ""),
      start=1, end = 6)

    froms = data.frame(
      matrix(
        data = c("a", "b"),
        ncol = 2, byrow = T)[NULL, ],
      stringsAsFactors = F)
    colnames(froms) =
      paste(c("reg_code", "reg_name"), rep(colnamecandidates[1], 2), sep = "_")

    tos = data.frame(
      matrix(data = c("a", "b"),
             ncol = 2, byrow = T)[NULL, ],
      stringsAsFactors = F)
    colnames(tos) = paste(c("reg_code", "reg_name"), rep(colnamecandidates[2], 2), sep = "_")

    fromstos = dplyr::bind_cols(dplyr::as_tibble(froms), dplyr::as_tibble(tos))
  } else {fromstos = NULL}


  return(fromstos)
}
regupdated = function(files){
  regiondef = region_at_time_txtfls(filename = files[1])

  for (i in 2:length(files)){
    print(i)
    regupdate = region_at_time_txtfls(filename = files[i])
    ## !! coming left_join: It would be best to find a way to join only by the "reg_code_x" variables but I could not find how to type this :-(
    both = dplyr::left_join(regiondef, regupdate)
    head(both)
    regupnames <- names(both)
    regupnamesl <- length(regupnames)
    both %>%
      dplyr::mutate(.,
                    !!sym(regupnames[regupnamesl-1]) :=
                      dplyr::case_when(
                        !is.na(!!dplyr::sym(regupnames[regupnamesl-1])) ~
                          !!dplyr::sym(regupnames[regupnamesl-1]),
                        TRUE ~ !!dplyr::sym(regupnames[regupnamesl-3])),
                    !!sym(regupnames[regupnamesl]) :=
                      dplyr::case_when(
                        !is.na(!!dplyr::sym(regupnames[regupnamesl])) ~ !!dplyr::sym(regupnames[regupnamesl]),
                        TRUE ~ !!dplyr::sym(regupnames[regupnamesl-2]))
      )  -> regiondef
  }
  return(regiondef)
}

no.regiontabell.flk = function(){
  #Fylker: https://www.ssb.no/en/klass/klassifikasjoner/104/versjon/1158/koder

  files <-   list.files( path = "./data-raw", pattern = ".txt", full.names = T) %>%
    .[which(!stringr::str_detect(., "~"))] %>%
    .[which(stringr::str_detect(., "Regindeling_Fylker"))]
  inndeling <- regupdated(files = files)
  return(inndeling)
}


no.regiontabell.kmn = function(){
  #Kommuner: https://www.ssb.no/en/klass/klassifikasjoner/131
  files <-   list.files(path = "./data-raw", pattern = ".txt", full.names = T) %>%
    .[which(!stringr::str_detect(., "~"))] %>%
    .[which(stringr::str_detect(., "Regindeling_Kommuner"))]
  inndeling <- regupdated(files = files)
  return(inndeling)

}

regref_fylke <- no.regiontabell.flk()
regref_kommune <- no.regiontabell.kmn()

usethis::use_data(regref_kommune, regref_fylke, overwrite = T)
